# Turns images found from a directory into GIFs
# A GIF consists of files that had same "prefix", delimeter being "-"
# image-1.png, image-2.png would turn into a image.gif
# 
# Note: overwrites existing files without asking
#
# In Windows use PowersSell 
# python -m pip install imageio
import imageio
import os
import sys
import getopt

# Instructions
help = 'gif_maker.py -i <inputdirectory> -o <outputdirectory>'
   
# Look for .png files
ext = '.png'

# Duration per frame in seconds
duration = 0.5
 
def get_files_recursively(start_directory, filter_extension=None):
    
    for root, _, files in os.walk(start_directory):
        for file in files:
            if filter_extension is None or file.lower().endswith(filter_extension):
                yield os.path.join(root, file)
        
            
def get_directory_images(source, file_extension=None):
    images = {}
    for file in get_files_recursively(source, file_extension):
        filename = file.split("\\")[-1]
        filenameParts = filename.split('-')
        # No delimiter, no GIF
        if filenameParts[0] == filename:
            continue;
        else:
            prefix = filenameParts[0]
            
        if images.get(prefix) is None:
            images[prefix] = []            
        
        images[prefix].append(file)
        
    return images 

def confirm_or_exit():
     while True:
        confirm = input('Proceed? [y]Yes or [n]No: ')
        if confirm in ('y', 'n'):
            if confirm == 'n':
                print('> exit')
                exit()
            else: 
                return
        else:
            print("\n Invalid Option. Please Enter a Valid Option.")
    


def main(argv):

    # Read input and output directory from arguments
    inputDirectory = ''
    outputDirectory = ''
    try:
        opts, args = getopt.getopt(argv,"hi:o:",["ifile=","ofile="])
    except getopt.GetoptError:
        print(help)
        sys.exit(2)
        
    if len(argv) < 1:
        print(help)
        sys.exit()
        
    for opt, arg in opts:
        if opt == '-h':
            print(help)
            sys.exit()
        elif opt in ("-i", "--idir"):
            inputDirectory = arg
        elif opt in ("-o", "--odir"):
            outputDirectory = arg
            
    print('Input directory is ' + inputDirectory)
    print('Output directory is ' + outputDirectory)
    
    # Create output directory if not exists
    if not os.path.exists(outputDirectory):
        print('Create output directory ' + outputDirectory)
        os.makedirs(outputDirectory)
    
    # Get images
    image_files = get_directory_images(inputDirectory, ext)
    print("Process " + str(len(image_files)) + " image set(s) into GIF(s):");
    for key, value in image_files.items() :
        print (os.path.join(outputDirectory, key + ".gif"))
    
    # Confirm 
    confirm_or_exit()
    
    # Turn files into data
    gif_frames = {}
    for set, image_files in image_files.items():
        
        # No one frame animations, thanks
        if len(image_files) == 1:
            continue; 
            
        for file in image_files:
            if gif_frames.get(set) is None:
                gif_frames[set] = []
            
            gif_frames[set].append(imageio.imread(file))
    
    # Save a gif per set 
    for setName, setFiles in gif_frames.items():      
        #for set in gif_frames
        outputFile = os.path.join(outputDirectory, setName + ".gif")
        print("Saving " + outputFile)
        imageio.mimsave(outputFile, setFiles, duration = duration)
        pass
    
    os.startfile(outputDirectory)
    
if __name__ == "__main__":
   main(sys.argv[1:])
